/* License of this file: CC0 Public Domain Dedication along
   see http://creativecommons.org/publicdomain/zero/1.0/ */

#include <stdio.h>
#include <stdlib.h>

#include <libmbtk/libmbtk.h>
#include <libmbtk/backend/sdl2.h>
#include <libmbtk/widget.h>
#include <libmbtk/style_f3d.h>

static char *font[] = {"../../../font/Vera.ttf,size=18"};
static mbtk_backend_t be;
static mbtk_display_t disp;

/* Initialize be and disp and set up f3d for default style. Exits on error. */
static void example_init_sdl2(int *argc, char **argv[])
{
	if (mbtk_sdl2_backend(&be) != 0) {
		fprintf(stderr, "Unable to set up the sdl2 backend\n");
		exit(1);
	}

	if (mbtk_be_init(&be) != 0) {
		fprintf(stderr, "Unable to initialize the sdl2 backend\n");
		exit(1);
	}

	if (mbtk_display_init(&be, &disp, argc, argv) != 0) {
		fprintf(stderr, "Unable to initialize mbtk display with the sdl2 backend\n");
		exit(1);
	}

	/* Set default style and font ID for the display */
	mbtk_f3d_style(&disp);
	disp.default_fid = mbtk_resolve_font(&disp, 1, font);
}

static mbtk_event_handled_t menu_cb(mbtk_widget_t *w, mbtk_kw_t id, void *user_data)
{
	const char *which = (const char *)user_data;
	printf("File menu clicked: '%s'\n", which);
	return MBTK_EVENT_HANDLED;
}

static mbtk_event_handled_t menu_toggle_cb(mbtk_widget_t *w, mbtk_kw_t id, void *user_data)
{
	printf("toggled: %d\n", mbtk_menu_stdrow_checkbox_toggle((mbtk_box_t *)w));
	return MBTK_EVENT_HANDLED;
}

static mbtk_event_handled_t menu_quit_cb(mbtk_widget_t *w, mbtk_kw_t id, void *user_data)
{
	mbtk_window_t *win = (mbtk_window_t *)user_data;
	mbtk_window_close(win);
	return MBTK_EVENT_HANDLED;
}

static mbtk_event_handled_t button_clk(mbtk_widget_t *w, mbtk_kw_t id, void *user_data)
{
	mbtk_menu_t *ctxmenu = user_data;
	mbtk_menu_open_popup(ctxmenu, w->win, w->win->x + w->hvbox.x + w->hvbox.w, w->win->y + w->hvbox.y + w->hvbox.h);
	return 0;
}


int main(int argc, char *argv[])
{
	mbtk_window_t *win;
	mbtk_box_t *root;
	mbtk_button_t *btn;
	mbtk_menu_bar_t *mb;
	mbtk_menu_t *menu, *ctxmenu;
	mbtk_box_t *mimp, *mexp, *m;

	example_init_sdl2(&argc, &argv);

	/* create a small, normal (non-modal, non-menu) window. -1 for coord means
	   we do not have a request on placement coords; the last two are recommended
	   initial window size */
	win = mbtk_window_new(&disp, MBTK_WNTY_NORMAL, "miniboxtk hello world", -1, -1, 150, 50);
	if (win == NULL) {
		fprintf(stderr, "Unable to create top window\n");
		exit(1);
	}

	/* create the root widget as a vbox so child widgets are packed vertically;
	   default alignment is "justify", which means excess space are put between
	   the widgets. */
	root = mbtk_vbox_new(NULL);
	mbtk_window_set_root_widget(win, &root->w);

	mb = mbtk_menu_bar_new(NULL);
	mbtk_box_add_widget(root, &mb->w, 0);

	mbtk_menu_bar_append_label(mb, "File", (menu = mbtk_menu_new(NULL)));
		m = mbtk_menu_append_label(menu, "Save"); mbtk_menu_stdrow_callback_invoked(m, menu_cb, "save something");
		m = mbtk_menu_append_label(menu, "Load"); mbtk_menu_stdrow_callback_invoked(m, menu_cb, "load something");
		mbtk_menu_append_sep(menu);
		mimp = mbtk_menu_append_label_submenu(menu, "Import");
		mexp = mbtk_menu_append_label_submenu(menu, "Export");
		mbtk_menu_append_sep(menu);
		m = mbtk_menu_append_label_full(menu, 1, "Bugfree", "b f"); mbtk_menu_stdrow_callback_invoked(m, menu_toggle_cb, NULL);
		m = mbtk_menu_append_label(menu, "Quit"); mbtk_menu_stdrow_callback_invoked(m, menu_quit_cb, win);

	mbtk_menu_bar_append_label(mb, "Edit", (menu = mbtk_menu_new(NULL)));
		mbtk_menu_append_label(menu, "Copy");
		mbtk_menu_append_label(menu, "Paste");

	mbtk_menu_bar_append_label(mb, "Help", (menu = mbtk_menu_new(NULL)));
		mbtk_menu_append_label(menu, "About");
		mbtk_menu_append_label(menu, "User manual");

	mbtk_menu_item_attach_submenu(&mimp->w, (menu = mbtk_menu_new(NULL)));
		m = mbtk_menu_append_label(menu, "Import foo"); mbtk_menu_stdrow_callback_invoked(m, menu_cb, "importing foo");
		m = mbtk_menu_append_label(menu, "Import bar"); mbtk_menu_stdrow_callback_invoked(m, menu_cb, "importing bar");

	mbtk_menu_item_attach_submenu(&mexp->w, (menu = mbtk_menu_new(NULL)));
		m = mbtk_menu_append_label(menu, "Export foo"); mbtk_menu_stdrow_callback_invoked(m, menu_cb, "exporting foo");
		m = mbtk_menu_append_label(menu, "Export bar"); mbtk_menu_stdrow_callback_invoked(m, menu_cb, "exporting bar");

	/* Create a context menu that we are going to use as a popup */
	ctxmenu = mbtk_menu_new(NULL);
	mbtk_menu_append_label(ctxmenu, "doesn't do anything");
	mbtk_menu_append_label(ctxmenu, "neither does this");

	/* Create and add a button for the context popup test */
	btn = mbtk_button_new_with_label("context\npopup!");
	mbtk_box_add_widget(root, &btn->w, 0);
	mbtk_button_callback_click(btn, button_clk, ctxmenu);


	/* main loop: run the GUI until win is closed */
	mbtk_run_window(win);

	/* Uninit everyhting at once */
	mbtk_uninit();
}
