/* License of this file: CC0 Public Domain Dedication along
   see http://creativecommons.org/publicdomain/zero/1.0/ */

#include <stdio.h>
#include <stdlib.h>

#include <libmbtk/libmbtk.h>
#include <libmbtk/backend/sdl2.h>
#include <libmbtk/widget.h>
#include <libmbtk/style_f3d.h>

static char *font[] = {"../../../font/Vera.ttf,size=18"};
static mbtk_backend_t be;
static mbtk_display_t disp;

/* Initialize be and disp and set up f3d for default style. Exits on error. */
static void example_init_sdl2(int *argc, char **argv[])
{
	if (mbtk_sdl2_backend(&be) != 0) {
		fprintf(stderr, "Unable to set up the sdl2 backend\n");
		exit(1);
	}

	if (mbtk_be_init(&be) != 0) {
		fprintf(stderr, "Unable to initialize the sdl2 backend\n");
		exit(1);
	}

	if (mbtk_display_init(&be, &disp, argc, argv) != 0) {
		fprintf(stderr, "Unable to initialize mbtk display with the sdl2 backend\n");
		exit(1);
	}

	/* Set default style and font ID for the display */
	mbtk_f3d_style(&disp);
	disp.default_fid = mbtk_resolve_font(&disp, 1, font);
}

static mbtk_event_handled_t button_clk(mbtk_widget_t *w, mbtk_kw_t id, void *user_data)
{
	mbtk_tabbed_t *tabbed = user_data; /* user_data is what mbtk_button_callback_click() had as 3rd parameter */
	fprintf(stderr, "Currently active page:\n");
	return 0;
}


static void add_page(mbtk_tabbed_t *tabbed, int id)
{
	mbtk_box_t *page;
	mbtk_label_t *lab;
	mbtk_button_t *btn;
	char tmp[128];

	page = mbtk_vbox_new(NULL);

	sprintf(tmp, "This is page number %d.", id);
	lab = mbtk_label_new(NULL, tmp);
	mbtk_box_add_widget(page, &lab->w, 0);

	/* Create an add a button */
	btn = mbtk_button_new_with_label("Click me!");
	mbtk_button_callback_click(btn, button_clk, tabbed);
	mbtk_box_add_widget(page, &btn->w, 0);

	sprintf(tmp, "Page #%d", id);
	mbtk_tabbed_append_page_with_label(tabbed, tmp, &page->w);
}

int main(int argc, char *argv[])
{
	mbtk_window_t *win;
	mbtk_tabbed_t *root;
	int n;

	example_init_sdl2(&argc, &argv);

	/* create a small, normal (non-modal, non-menu) window. -1 for coord means
	   we do not have a request on placement coords; the last two are recommended
	   initial window size */
	win = mbtk_window_new(&disp, MBTK_WNTY_NORMAL, "miniboxtk hello world", -1, -1, 150, 50);
	if (win == NULL) {
		fprintf(stderr, "Unable to create top window\n");
		exit(1);
	}

	/* The whole window will be a tabbed widget */
	root = mbtk_tabbed_new(NULL, MBTK_TABBED_TOP);
	mbtk_window_set_root_widget(win, &root->w);

	/* Create a few pages */
	for(n = 0; n < 4; n++)
		add_page(root, n);

	/* main loop: run the GUI until win is closed */
	mbtk_run_window(win);

	/* Uninit everyhting at once */
	mbtk_uninit();
}
