/* License of this file: CC0 Public Domain Dedication along
   see http://creativecommons.org/publicdomain/zero/1.0/ */

#include <stdio.h>
#include <stdlib.h>

#include <libmbtk/libmbtk.h>
#include <libmbtk/backend/sdl2.h>
#include <libmbtk/widget.h>
#include <libmbtk/style_f3d.h>

static char *font[] = {"../../../font/Vera.ttf,size=18"};
static mbtk_backend_t be;
static mbtk_display_t disp;

/* Initialize be and disp and set up f3d for default style. Exits on error. */
static void example_init_sdl2(int *argc, char **argv[])
{
	if (mbtk_sdl2_backend(&be) != 0) {
		fprintf(stderr, "Unable to set up the sdl2 backend\n");
		exit(1);
	}

	if (mbtk_be_init(&be) != 0) {
		fprintf(stderr, "Unable to initialize the sdl2 backend\n");
		exit(1);
	}

	if (mbtk_display_init(&be, &disp, argc, argv) != 0) {
		fprintf(stderr, "Unable to initialize mbtk display with the sdl2 backend\n");
		exit(1);
	}

	/* Set default style and font ID for the display */
	mbtk_f3d_style(&disp);
	disp.default_fid = mbtk_resolve_font(&disp, 1, font);
}

int main(int argc, char *argv[])
{
	mbtk_window_t *win;
	mbtk_tbl_t *root;
	const char **s, *strs[] = { "first", "second", "third", "fourth", "fifth", "sixth", "7\nth", "eighth", "nineth", "orphan", NULL };
	int *f, fill[] =          { 0,       1,        0,       0,        0,       0,       0,       0,        0,        0,        0 };
	int *af, alt_fill[] =     { 0,       0,        0,       1,        0,       0,       0,       0,        0,        0,        0 };

	example_init_sdl2(&argc, &argv);

	/* create a small, normal (non-modal, non-menu) window. -1 for coord means
	   we do not have a request on placement coords; the last two are recommended
	   initial window size */
	win = mbtk_window_new(&disp, MBTK_WNTY_NORMAL, "miniboxtk hello world", -1, -1, 150, 50);
	if (win == NULL) {
		fprintf(stderr, "Unable to create top window\n");
		exit(1);
	}

	/* create the root widget as a vbox so child widgets are packed vertically;
	   default alignment is "justify", which means excess space are put between
	   the widgets. */
	root = mbtk_tbl_new(NULL, 3);
	root->bg_even_odd = 1;
	root->bg_first = 1;
	mbtk_window_set_root_widget(win, &root->w);

	/* Create and add a labels */
	for(s = strs, f = fill, af = alt_fill; *s != NULL; s++,f++,af++) {
		mbtk_label_t *lab = mbtk_label_new(NULL, *s);
		lab->halign = -1;
		if (*f)  mbtk_label_set_span(lab, HVBOX_FILL);
		if (*af) mbtk_label_set_alt_span(lab, HVBOX_FILL);
		mbtk_tbl_add_widget(root, &lab->w, 0);
	}

	/* main loop: run the GUI until win is closed */
	mbtk_run_window(win);

	/* Uninit everyhting at once */
	mbtk_uninit();
}
