/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#define INCLUDE_THREADS
#include "../../config.h"

#ifdef MBTK_HAVE_THREAD_PTHREAD

typedef struct mbtk_be_thread_s {
	pthread_t thread;
} mbtk_be_thread_t;

typedef int mbtk_be_sem_t;
#define MBTK_BE_INVALID_SEM -1

#endif

#ifdef MBTK_HAVE_THREAD_WIN32

typedef struct mbtk_be_thread_s {
	HANDLE thread;
	void *(*func)(void *);
	void *udata;
} mbtk_be_thread_t;

typedef HANDLE mbtk_be_sem_t;
#define MBTK_BE_INVALID_SEM 0

#endif



#ifdef MBTK_BE_INVALID_SEM
/* Create and run new thread. Return 0 on success. */
int mbtk_be_thread_create(mbtk_be_thread_t *thr, void *(*func)(void *), void *udata);

/* Blocking wait until thr exits */
int mbtk_be_thread_wait(mbtk_be_thread_t *thr);

/* Create a semaphore with an initial counter value. Returns
   MBTK_BE_INVALID_SEM on error */
mbtk_be_sem_t mbtk_be_semget(int init_cnt);

/* Remove a semaphore - can't fail */
void mbtk_be_semdel(mbtk_be_sem_t sem);

/* Decrease semaphore value. If block is true, wait until decreasing is
   possible. If block is false and decreasing is not possible, fail.
   Returns 0 on success. */
int mbtk_be_semwait(mbtk_be_sem_t sem, int block);

/* Increase semaphore value. Returns 0 on success. */
int mbtk_be_sempost(mbtk_be_sem_t sem);
#endif
