/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#include <ctype.h>

#define SKIPEP    while((*curr == '=') || (*curr == ',') || (isspace(*curr))) curr++

static glfw_font_t *glfw_resolve_font(glfw_display_t *sd, const char *cfg)
{
	char *name, *curr;
	int size = 14, res, len;
	glfw_font_t *f;

	curr = strchr(cfg, ',');
	if (curr == NULL)
		len = strlen(cfg);
	else
		len = curr - cfg;

	name = malloc(len+1);
	memcpy(name, cfg, len);
	name[len] = '\0';

	while(curr != NULL) {
		SKIPEP;
		if (*curr == '\0') break;
		if (strncmp(curr, "size", 4) == 0) {
			curr += 4;
			SKIPEP;
			size = strtol(curr, &curr, 10);
		}
	}


	f = malloc(sizeof(glfw_font_t));
	f->size = size;
	res = FT_New_Face(sd->lib, name, 0, &f->face);
	free(name);

	if (res != 0) {
		free(f);
		return NULL;
	}

	FT_Set_Char_Size(f->face, (double)size*32.0*1.333, 0, 100, 0);

	f->asc = ((f->face->ascender+32) / 64)/2;
	f->dsc = -((f->face->descender+32) / 64)/2;

	return f;
}

static mbtk_font_id_t mbtk_glfw_resolve_font(mbtk_display_t *display, int argc, char *argv[])
{
	int n;
	mbtk_font_id_t res;
	glfw_display_t *gd = (glfw_display_t *)display;

	for(n = 0; n < argc; n++) {
		glfw_font_t *f = glfw_resolve_font(gd, argv[n]);
		if (f != NULL) {
			res.p = f;
			return res;
		}
	}

	res.l = -1;
	return res;
}

