/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

static int mbtk_glfw_proc_stdin(glfw_display_t *gd, mbtk_event_t *dst)
{
	mbtk_display_t *disp = (mbtk_display_t *)gd;
	dst->type = MBTK_EV_RSTDIN;
	dst->win = disp->topwin;
	dst->data.rstdin.len = iot_stdio.len;
	dst->data.rstdin.data = iot_stdio.data;
	iot_stdio.loaded = 0;

/*				printf("STDIO EVENT %d '%s'!\n", iot_stdio.len, iot_stdio.data);*/

	mbtk_be_semwait(iot_stdio.enable, 0); /* decrease to 0 to make sure the therad waits before sending the next event */
	if (!iot_stdio.disabled)
		mbtk_be_sempost(iot_stdio.enable); /* enable sending the next event */
	mbtk_be_sempost(iot_stdio.handled); /* release data */
	return 1;
}

static int mbtk_glfw_proc_fdwatch(glfw_display_t *gd, mbtk_event_t *dst)
{
	mbtk_display_t *disp = (mbtk_display_t *)gd;
	dst->type = MBTK_EV_FDWATCH;
	dst->win = disp->topwin;
	dst->data.fdwatch.fd = gd->iot_fdwatch.fd;
	dst->data.fdwatch.readable = gd->iot_fdwatch.readable;
	dst->data.fdwatch.writable = gd->iot_fdwatch.writable;
	dst->data.fdwatch.closed = gd->iot_fdwatch.closed;

/*			printf("FDWATCH event fd=%d\n", dst->data.fdwatch.fd); */

	/* release data */
	gd->iot_fdwatch.loaded = 0;
	mbtk_be_sempost(gd->iot_fdwatch.handled);
	return 1;
}

static int mbtk_glfw_proc_queue(glfw_display_t *gd, mbtk_event_t *dst)
{
	double ts = gd->evq->timestamp;
	memcpy(dst, &gd->evq->ev, sizeof(mbtk_event_t));
	free_ev_first(gd);
	
	/* try to batch unicode with key release */
	if (dst->type == MBTK_EV_KEY_RELEASE)
		glfwWaitEventsTimeout(0.1);
	if ((gd->evq != NULL) && (dst->type == MBTK_EV_KEY_RELEASE) && (gd->evq->ev.type == MBTK_EV_KEY_RELEASE) && (gd->evq->timestamp - ts < 0.1)) {
		if ((dst->data.key.edit == 0) && (gd->evq->ev.data.key.edit != 0)) {
			dst->data.key.edit = gd->evq->ev.data.key.edit;
			free_ev_first(gd);
		}
		else if ((dst->data.key.edit != 0) && (gd->evq->ev.data.key.edit == 0)) {
			dst->data.key.scancode = gd->evq->ev.data.key.scancode;
			dst->data.key.mods = gd->evq->ev.data.key.mods;
			dst->data.key.sym = gd->evq->ev.data.key.sym;
			free_ev_first(gd);
		}
	}
	return 1;
}


static int mbtk_glfw_iterate(mbtk_display_t *display, mbtk_event_t *dst, double timeout, int block)
{
	glfw_display_t *gd = (glfw_display_t *)display;

	do {
		if (gd->evq == NULL) { /* no event, try to get some */
			if ((timeout > 0) || block)
				glfwWaitEventsTimeout(timeout);
			else
				glfwPollEvents();
		}

		/* Because of the impotent event API, we don't get a callback or
		   any notification on background thread events. Need to check them
		   manually. */
		if (iot_stdio.loaded)
			return mbtk_glfw_proc_stdin(gd, dst);

		if (gd->iot_fdwatch.loaded)
			return mbtk_glfw_proc_fdwatch(gd, dst);

		/* Process the event queue created via the callbacks: pop first event */
		if (gd->evq != NULL)
			return mbtk_glfw_proc_queue(gd, dst);

	} while(block);

	return 0;
}
