/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#define GL_GLEXT_PROTOTYPES 1
#include <GL/gl.h>
#include <GLFW/glfw3.h>
#include <libmbtk/backend.h>
#include <libmbtk/util.h>
#include "vtvx.h"
#include "vtpr.h"
#include <genvector/vtl0.h>
#include <genlist/gendlist.h>
#include "../common/threads.h"
#include <ft2build.h>
#include FT_FREETYPE_H


typedef struct glfw_font_s {
	FT_Face face;
	int size; /* user requested size */
	int asc, dsc;
} glfw_font_t;

typedef struct mbtk_glfw_event_s mbtk_glfw_event_t;
struct mbtk_glfw_event_s {
	mbtk_event_t ev;
	double timestamp;
	mbtk_glfw_event_t *next; /* singly linked list in the per display event queue */
};

typedef struct glfw_win_s glfw_win_t;

typedef struct glfw_display_s {
	FT_Library lib;
	uall_slabs_t evs;        /* event allocation */
	mbtk_glfw_event_t *evq;  /* event queue, head (NULL if empty) */
	mbtk_glfw_event_t *evq_tail;  /* event queue, tail (last event appended) */
	double base_time;

	struct {
		/* output */
		int fd;
		unsigned readable:1;
		unsigned writable:1;
		unsigned closed:1;
		unsigned loaded:1;

		/* thread */
		unsigned thread_inited:1;
		mbtk_be_thread_t tid;
		mbtk_be_sem_t handled;       /* main to thread: 0: thread's waiting non-blocking after a read; when main finishes picking up it increases it back to 1 to start the next read */
		int ctrlw, ctrlr;       /* control pipe fds; w is write from main, r is read on the thread's side */
	} iot_fdwatch;

} glfw_display_t;

struct glfw_win_s {
	GLFWwindow *window;
	GLfloat red, green, blue, alpha;
	glfw_font_t *font;
	GLuint program, inputColor_location, inputTexture_location, xform_location, position_buffer;
	float vertbuf_last_r, vertbuf_last_g, vertbuf_last_b, vertbuf_last_a;
	vtvx_t vxbuf;
	vtpr_t prbuf;
	long last_mouse_x, last_mouse_y;
	glfw_win_t *last_mouse_subw;

	gdl_list_t subwins;
	gdl_elem_t link;
	unsigned subwin:1;
};

