/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

static void mbtk_glfw_win(mbtk_window_type_t type)
{
/* should call this, but leads to a segfault upon first draw after expose
	glfwDefaultWindowHints();
*/

	glfwWindowHint(GLFW_OPENGL_API, 1);

/* no idea how to enable vulkan
	if (type & MBTK_WNTY_OPENGL)       
	else if (type & MBTK_WNTY_VULKAN)
*/

	switch(type & 0xff) {
		case MBTK_WNTY_NORMAL:
			glfwWindowHint(GLFW_RESIZABLE, 1);
			glfwWindowHint(GLFW_DECORATED, 1);
			break;
		case MBTK_WNTY_MODAL:
			glfwWindowHint(GLFW_RESIZABLE, 1);
			glfwWindowHint(GLFW_DECORATED, 1);
			break;
		case MBTK_WNTY_DROPDOWN_MENU:
			glfwWindowHint(GLFW_RESIZABLE, 0);
			glfwWindowHint(GLFW_DECORATED, 0);
			glfwWindowHint(GLFW_FLOATING, 1);
			break;
		case MBTK_WNTY_POPUP_MENU:
			glfwWindowHint(GLFW_RESIZABLE, 0);
			glfwWindowHint(GLFW_DECORATED, 0);
			glfwWindowHint(GLFW_FLOATING, 1);
			break;
		case MBTK_WNTY_TEAROFF_MENU:
			glfwWindowHint(GLFW_RESIZABLE, 1);
			glfwWindowHint(GLFW_DECORATED, 1);
			glfwWindowHint(GLFW_FLOATING, 1);
			break;
		case MBTK_WNTY_COMBO:
			glfwWindowHint(GLFW_RESIZABLE, 0);
			glfwWindowHint(GLFW_DECORATED, 0);
			glfwWindowHint(GLFW_FLOATING, 1);
			break;
		case MBTK_WNTY_TOOLTIP:
			glfwWindowHint(GLFW_RESIZABLE, 0);
			glfwWindowHint(GLFW_DECORATED, 0);
			glfwWindowHint(GLFW_FLOATING, 1);
			break;
	}
}

static mbtk_window_t *mbtk_glfw_window_new(mbtk_display_t *disp, mbtk_window_type_t type, const char *title, long x, long y, long w, long h)
{
	mbtk_window_t *win = calloc(sizeof(mbtk_window_t), 1);
	glfw_win_t *gw = (glfw_win_t *)win;

	if ((w < 0) || (h < 0))
		w = h = 100;

	mbtk_glfw_win(type);
	gw->window = glfwCreateWindow(w, h, title, NULL, NULL);
	if (gw->window == NULL)
		goto error;

	glfwSetWindowUserPointer(gw->window, gw);

	if ((x >= 0) && (y >= 0))
		glfwSetWindowPos(gw->window, x, y);

	glfwMakeContextCurrent(gw->window);

	mbtk_glfw_win_event_setup(gw);

	gw->last_mouse_x = gw->last_mouse_y = -1;

	init_checkver();
	draw_win_init(gw);

	return win;

	error:;
	free(win);
	return NULL;
}

static mbtk_window_t *mbtk_glfw_subwindow_new(mbtk_display_t *disp, mbtk_widget_t *parent, long x, long y, long w, long h)
{
	glfw_win_t *parent_win = (glfw_win_t *)parent->win;
	mbtk_window_t *win = calloc(sizeof(mbtk_window_t), 1);
	glfw_win_t *gw;
	
	gw = (glfw_win_t *)win;
	gw->subwin = 1;
	gdl_append(&parent_win->subwins, gw, link);
	return win;
}

static void mbtk_glfw_window_close(mbtk_window_t *win)
{
	glfw_win_t *gw = (glfw_win_t *)win;

	if (!gw->subwin) {
		draw_win_uninit(gw);
		glfwDestroyWindow(gw->window);
	}
	else {
		glfw_win_t *parent_win = (glfw_win_t *)win->in_widget->win;
		gdl_remove(&parent_win->subwins, gw, link);
		/* No need to call draw_win_init(gw): we use the host window's */
	}
}

static mbtk_window_t *mbtk_glfw_win2render_win(mbtk_window_t *win)
{
	return (win->in_widget) ? win->in_widget->render_win : win;
}

static void mbtk_glfw_window_set_title(mbtk_window_t *win, const char *title)
{
	glfw_win_t *gw = (glfw_win_t *)win;
	glfwSetWindowTitle(gw->window, title);
}

static void mbtk_glfw_window_raise(mbtk_window_t *win)
{
	glfw_win_t *gw = (glfw_win_t *)win;
	glfwShowWindow(gw->window);
	glfwFocusWindow(gw->window);
}

static void mbtk_glfw_window_hide(mbtk_window_t *win)
{
	glfw_win_t *gw = (glfw_win_t *)win;
	glfwHideWindow(gw->window);
}

static void mbtk_glfw_window_show(mbtk_window_t *win)
{
	glfw_win_t *gw = (glfw_win_t *)win;
	glfwShowWindow(gw->window);
}

static void mbtk_glfw_window_transient4(mbtk_window_t *win, mbtk_window_t *tr_for)
{
}

static void mbtk_glfw_window_set_min_size(mbtk_window_t *win, long w, long h)
{
	glfw_win_t *gw = (glfw_win_t *)win;
	glfwSetWindowSizeLimits(gw->window, w, h, GLFW_DONT_CARE, GLFW_DONT_CARE);
}

static void mbtk_glfw_window_set_size(mbtk_window_t *win, long w, long h)
{
	glfw_win_t *gw = (glfw_win_t *)win;
	glfwSetWindowSize(gw->window, w, h);
}


static int mbtk_glfw_window_get_size(mbtk_window_t *win, long *w, long *h)
{
	glfw_win_t *gw = (glfw_win_t *)win;
	int wi, hi;

	glfwGetWindowSize(gw->window, &wi, &hi);
	*w = wi;
	*h = hi;
	return 0;
}

