/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#ifndef LIBMBTK_SDL2_TEXT_H
#define LIBMBTK_SDL2_TEXT_H
#include <SDL.h>
#include <SDL_ttf.h>
#include <libmbtk/src_3rd/genht/htsp.h>

typedef struct mbtk_sdl2_font_ctx_s mbtk_sdl2_font_ctx_t;

typedef struct mbtk_sdl2_ttfcache_s {
	htsp_t *cache;
} mbtk_sdl2_ttfcache_t;

typedef struct mbtk_sdl2_font_s {
	TTF_Font *ttf;
	char *id;
	int size;
} mbtk_sdl2_font_t;

struct mbtk_sdl2_font_ctx_s {
	SDL_Renderer *renderer;
	mbtk_sdl2_ttfcache_t *font_cache;
	htsp_t *texture_cache;
	int ttf_solid; /* whether to draw ttf in solid (vs. antialiased) */
};

typedef struct mbtk_sdl2_text_s {
	mbtk_sdl2_font_t *font;
	char *id;
	char *message;     /* points into id */
	SDL_Color color;
	SDL_Texture *texture;
	int refco;
} mbtk_sdl2_text_t;

/* Initialize a font cache and load font files in it; each fontfile/size is
   a new font handle */
void mbtk_sdl2_font_init(mbtk_sdl2_ttfcache_t *font_cache);
void mbtk_sdl2_font_uninit(mbtk_sdl2_ttfcache_t *font_cache);
mbtk_sdl2_font_t *mbtk_sdl2_font_open(mbtk_sdl2_ttfcache_t *font_cache, const char *font_file, int size, int hinting, int style);

/* Init/uninit a text context for a renderer using a font cache */
mbtk_sdl2_font_ctx_t *mbtk_sdl2_font_ctx_init(SDL_Renderer *renderer, mbtk_sdl2_ttfcache_t *font_cache);
void mbtk_sdl2_font_ctx_uninit(mbtk_sdl2_font_ctx_t *ctx);

/* Create/destroy a bitmap text in the cache */
mbtk_sdl2_text_t *mbtk_sdl2_text_new(mbtk_sdl2_font_ctx_t *ctx, mbtk_sdl2_font_t *font, const char *message, SDL_Color color);
void mbtk_sdl2_text_destroy(mbtk_sdl2_font_ctx_t *ctx, mbtk_sdl2_text_t *txt);


/* Render a cached bitmap text at x;y */
void mbtk_sdl2_text_render(mbtk_sdl2_font_ctx_t *ctx, mbtk_sdl2_text_t *txt, int x, int y);

#endif
