/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

static int mbtk_test_draw_color(mbtk_window_t *win, int r, int g, int b, int a)
{
	test_win_t *tw = (test_win_t *)win;
	printf("draw_color: W%d, #%02x%02x%02x%02x\n", tw->id, r, g, b, a);
	return 0;
}

static void mbtk_test_draw_fillrect(mbtk_window_t *win, int x1, int y1, int x2, int y2)
{
	test_win_t *tw = (test_win_t *)win;
	printf("draw_fillrect: W%d, %d;%d %d;%d @%ld;%ld\n", tw->id, x1, y1, x2, y2, win->content_x0, win->content_y0);
}

static void mbtk_test_draw_line(mbtk_window_t *win, int x1, int y1, int x2, int y2)
{
	test_win_t *tw = (test_win_t *)win;
	printf("draw_line: W%d, %d;%d %d;%d @%ld;%ld\n", tw->id, x1, y1, x2, y2, win->content_x0, win->content_y0);
}

static void mbtk_test_draw_hline(mbtk_window_t *win, int x1, int y1, int x2)
{
	test_win_t *tw = (test_win_t *)win;
	printf("draw_hline: W%d, %d;%d %d;* @%ld;%ld\n", tw->id, x1, y1, x2, win->content_x0, win->content_y0);
}

static void mbtk_test_draw_vline(mbtk_window_t *win, int x1, int y1, int y2)
{
	test_win_t *tw = (test_win_t *)win;
	printf("draw_vline: W%d, %d;%d *;%d @%ld;%ld\n", tw->id, x1, y1, y2, win->content_x0, win->content_y0);

}

int mbtk_test_draw_set_font(mbtk_window_t *win, mbtk_font_id_t fontid)
{
	test_win_t *tw = (test_win_t *)win;
	printf("draw_set_font: W%d, F%ld\n", tw->id, fontid.l);
	return 0;
}

static void mbtk_test_draw_query_text(mbtk_window_t *win, int *wo, int *asco, int *desco, char *s, int slen)
{
	test_win_t *tw = (test_win_t *)win;

	*wo = slen * 8;
	*asco = 10;
	*desco = 6;

	printf("draw_query_text: W%d, '%s' %d -> %d %d %d\n", tw->id, s, slen, *wo, *asco, *desco);
}

static long mbtk_test_draw_query_text_pos(mbtk_window_t *win, long *char_start_px, long *char_end_px, long pxpos, long delta_chr, char *s, int slen)
{
	test_win_t *tw = (test_win_t *)win;
	long chidx = pxpos/8 + delta_chr;

	printf("draw_query_text_pos: W%d, '%s' %d at %ld %ld -> ", tw->id, s, slen, pxpos, delta_chr);

	if ((chidx < 0) || (chidx >= slen)) {
		printf("chidx out of range (%ld)\n", chidx);
		return -1;
	}

	*char_start_px = chidx * 8;
	*char_end_px = chidx * 8 + 7;

	printf("%ld, %ld..%ld\n", chidx, *char_start_px, *char_end_px);

	return chidx;
}


static void mbtk_test_draw_text(mbtk_window_t *win, int x, int y, char *s, int slen, void **cache)
{
	test_win_t *tw = (test_win_t *)win;
	printf("draw_text: W%d, %d;%d '%s' %d @%ld;%ld\n", tw->id, x, y, s, slen, win->content_x0, win->content_y0);
}

static int mbtk_test_draw_import_pixmap(mbtk_display_t *display, mbtk_pixmap_t *dst, unsigned char *pixels, int width, int height, int has_alpha)
{
	test_display_t *td = (test_display_t *)display;

	dst->bedata.ul = ++td->pxids;
	printf("draw_import_pixmap: %d;%d %d -> P%ld\n", width, height, has_alpha, dst->bedata.ul);

	return 0;
}

static void mbtk_test_draw_free_pixmap(mbtk_display_t *display, mbtk_pixmap_t *px)
{
	printf("draw_free_pixmap: P%ld\n", px->bedata.ul);
}

static mbtk_px_id_t mbtk_test_draw_bind_pixmap(mbtk_window_t *win, mbtk_pixmap_t *px)
{
	test_win_t *tw = (test_win_t *)win;
	mbtk_px_id_t pxid;
	pxid.l = px->bedata.ul;
	
	printf("draw_bind_pixmap: W%d, P%ld\n", tw->id, pxid.l);
	return pxid;
}

static void mbtk_test_draw_pixmap(mbtk_window_t *win, int x, int y, mbtk_px_id_t pxid)
{
	test_win_t *tw = (test_win_t *)win;

	printf("draw_pixmap: W%d, %d;%d, P%ld @%ld;%ld\n", tw->id, x, y, pxid.l, win->content_x0, win->content_y0);
}


static void mbtk_test_draw_flush(mbtk_window_t *win)
{
	test_win_t *tw = (test_win_t *)win;
	printf("draw_flush: W%d\n", tw->id);
}


static void mbtk_test_draw_start(mbtk_window_t *win)
{
	test_win_t *tw = (test_win_t *)win;
	printf("draw_start: W%d\n", tw->id);
}

