/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

static mbtk_font_id_t mbtk_xlib_resolve_font(mbtk_display_t *display, int argc, char *argv[])
{
	xlib_display_t *xd = (xlib_display_t *)display;
	int n, found;
	mbtk_font_id_t idx;
	XFontStruct *fi;

	for(found = n = 0; n < xd->fonts.used; n++) {
		if (xd->fonts.array[n] == NULL) {
			idx.l = n;
			found = 1;
			break;
		}
	}

	if (!found) {
		idx.l = xd->fonts.used;
		vtp0_append(&xd->fonts, NULL);
	}

	for(n = 0; n < argc; n++) {
		fi = XLoadQueryFont(xd->display, argv[n]);
		if (fi != NULL)
			break; /* stop at the first that worked */
	}

	/* fallback: find any font that already exists */
	if (fi == NULL) {
		for(found = n = 0; n < xd->fonts.used; n++) {
			if (xd->fonts.array[n] != NULL) {
				fi = xd->fonts.array[n];
				idx.l = n;
				return idx;
			}
		}
	}

	/* fallback: fixed font should work anywhere */
	if (fi == NULL)
		fi = XLoadQueryFont(xd->display, "fixed");

	/* fallback: return invalid */
	if (fi == NULL) {
		mbtk_error("mbtk_xlib fatal error: can not resolve _any_ font\n");
		idx.l = -1;
		return idx;
	}

	xd->fonts.array[idx.l] = fi;
	return idx;
}
