/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <libmbtk/backend.h>
#include <libmbtk/util.h>
#include <genvector/vtp0.h>
#include <libmbtk/src_3rd/genht/htip.h>
#include <libmbtk/src_3rd/genht/hash.h>

#include "../common/fdwatch.h"
#include "../common/monotime.h"

typedef unsigned long htuu_key_t;
typedef unsigned long htuu_value_t;
#define HT(x) htuu_ ## x
#include <libmbtk/src_3rd/genht/ht.h>
#include <libmbtk/src_3rd/genht/ht.c>

static int htuu_keyeq(htuu_key_t a, htuu_key_t b)
{
	return a == b;
}

static unsigned htuu_keyhash(htuu_key_t a)
{
	return longhash((long)a);
}

#undef HT


typedef struct xlib_win_s {
	Window window;
	GC gc;
	XFontStruct *fnt;
	int last_w, last_h, last_x, last_y;
	unsigned sub:1; /* set if window is a sub-window */
	unsigned skip_render:1; /* do not render anything in this window (caller will render) */
} xlib_win_t;

typedef struct xlib_display_s {
	Display *display;
	int screen, depth;
	Visual *visual;
	int fd;
	Colormap cmap;
	htuu_t clr_cache;
	vtp0_t fonts;     /* key: mbtk_font_id_t.l; value: (XFontStruct *) or NULL */
	htip_t id2win;    /* map X window ID to (xlib_win_t *) */
	struct {          /* cache atom lookup by name */
		Atom wmdel, wmactw;
		Atom wm_hints, wintype, wtype_dialog, wtype_ddmenu, wtype_pumenu;
		Atom wtype_menu, wtype_combo, wtype_ttip, wmst_modal, wmstate;
		Atom clipboard, selection, utf8_str, comp_str, targets, savetargets;
		Atom null, mult, pair;
	} xa;
	struct {          /* cache key codes for modifiers */
		KeyCode lshift, rshift;
		KeyCode lctrl, rctrl;
		KeyCode lalt, ralt;
	} xk;
	mbtk_watch_fd_t watch;
	double base_time;
	char *clipboard;

	/* keyboard state */
	unsigned int num_lock_mask;
	unsigned num_lock:1;
	unsigned caps_lock:1;
} xlib_display_t;
