/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#ifndef LIBMBTK_DEBUG_INLINES_H
#define LIBMBTK_DEBUG_INLINES_H

/* API: Calls useful for debugging the hvbox layout  */

#include <libmbtk/config.h>
#include <libmbtk/widget.h>
#include <stdio.h>
#include <libmbtk/src_3rd/genhvbox/hvbox_debug.h>

/* name is not copied or free'd */
MBTK_INLINE void mbtk_dbg_widget_set_name(mbtk_widget_t *w, const char *name);

/* Print the hvbox tree of the whole window w is in */
MBTK_INLINE void mbtk_dbg_widget_dump_tree(FILE *dst, mbtk_widget_t *w);

/* Print the hvbox subtree starting from w */
MBTK_INLINE void mbtk_dbg_widget_dump_subtree(FILE *dst, mbtk_widget_t *w);

/*** Implementation ***/

MBTK_INLINE void mbtk_dbg_widget_set_name(mbtk_widget_t *w, const char *name)
{
	w->hvbox.widget = (char *)name;
}

MBTK_INLINE void mbtk_dbg_widget_dump_tree(FILE *dst, mbtk_widget_t *w)
{
	mbtk_window_t *win = w->win;
	hvbox_dump_nodes(dst, 0, win->root, 1);
}

MBTK_INLINE void mbtk_dbg_widget_dump_subtree(FILE *dst, mbtk_widget_t *w)
{
	hvbox_dump_nodes(dst, 0, &w->hvbox, 1);
}

#endif
