/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#include <stdlib.h>
#include <string.h>
#include "style.h"
#include "event.h"
#include "libmbtk.h"
#include "display.h"
#include "mpm.h"
#include "timer.h"

int mbtk_display_init(mbtk_backend_t *be, mbtk_display_t *disp, int *argc, char **argv[])
{
	int res;

	memset(&disp->windows, 0, sizeof(disp->windows));
	disp->topwin = NULL;
	disp->need_win_redraw = 0;
	disp->win_next_redraw = 0;
	disp->be = NULL;
	disp->styleprop = NULL;
	memset(&disp->link, 0, sizeof(disp->link));
	vtp0_init(&disp->timers);
	disp->timerq = NULL;

	if (be->display_init == NULL)
		return -1;

	res = be->display_init(be, disp, argc, argv);
	if (res == 0)
		disp->be = be;

	mbtk_event_init_display(disp);
	mbtk_mpm_init_display(disp);

	if (res == 0)
		gdl_append(&be->displays, disp, link);

	return res;
}

void mbtk_display_uninit(mbtk_backend_t *be, mbtk_display_t *disp)
{
	mbtk_window_t *win;

	mbtk_timer_uninit_display(disp);

	for(win = gdl_first(&disp->windows); win != NULL; win = gdl_first(&disp->windows))
		mbtk_window_close(win);

	gdl_remove(&be->displays, disp, link);
	mbtk_mpm_uninit_display(disp);
	if (be->display_uninit != NULL)
		be->display_uninit(be, disp);
	disp->be = NULL;
	free(disp->styleprop);
	disp->styleprop = NULL;
}

void mbtk_clipboard_set(mbtk_display_t *disp, const char *new_content)
{
	if ((disp == NULL) || (disp->be == NULL) || (disp->be->clipboard_set == NULL))
		return;

	disp->be->clipboard_set(disp, new_content);
}

char *mbtk_clipboard_get(mbtk_display_t *disp)
{
	if ((disp == NULL) || (disp->be == NULL) || (disp->be->clipboard_get == NULL))
		return NULL;

	return disp->be->clipboard_get(disp);
}

void mbtk_clipboard_free(mbtk_display_t *disp, char *str_got)
{
	if ((disp == NULL) || (disp->be == NULL) || (disp->be->clipboard_free == NULL))
		return;

	disp->be->clipboard_free(disp, str_got);
}


double mbtk_get_time_el(mbtk_display_t *disp)
{
	if ((disp == NULL) || (disp->be == NULL) || (disp->be->get_time_el == NULL))
		return -1;

	return disp->be->get_time_el(disp);
}


void mbtk_display_set_topwin(mbtk_display_t *disp, mbtk_window_t *win)
{
	if ((disp != NULL) && (disp->be != NULL) && (disp->be->set_topwin != NULL))
		disp->be->set_topwin(disp, win);
	disp->topwin = win;
}

void mbtk_display_redraw_windows(mbtk_display_t *disp)
{
	mbtk_window_t *win;
	for(win = gdl_first(&disp->windows); win != NULL; win = gdl_next(&disp->windows, win))
		mbtk_win_redraw(win);
	disp->need_win_redraw = 0;
}

mbtk_keymod_t mbtk_display_query_kbd_mods(mbtk_display_t *disp)
{
	if ((disp != NULL) && (disp->be != NULL) && (disp->be->query_kbd_mods != NULL))
		return disp->be->query_kbd_mods(disp);
	return 0;
}
