/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#ifndef LIBMBTK_HANDLER_H
#define LIBMBTK_HANDLER_H

/* API: widget handlers react on events; used by widget implementation */


#include <libmbtk/global_typedefs.h>
#include <libmbtk/src_3rd/genlist/gendlist.h>
struct mbtk_handler_s {
	const char *name;                                        /* cookie; a handler implementation will always use the same pointer for name so it can be checked with ==; also useful for debugging */
	mbtk_event_handled_t (*event)(mbtk_widget_t *widget, mbtk_event_t *ev);   /* called when an event is received for a widget (or child of that widget) that has the handler associated; if event is not handled it is propagated up in the widget hierarchy */
	void (*widget_remove)(mbtk_widget_t *widget);            /* called when one of the widgets of the handler is destroyed or for other reason the widget should be removed from the handler */
	void (*handler_destroy)(mbtk_handler_t *hand);           /* called when the parent window of the handler is destroyed */
	gdl_elem_t link;                                         /* linked list of handlers in a window (for destroying handlers when window is destroyed); only when dynamically allocated */
	/* over-allocated with handler specific fields */
};


#endif
