/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#include "window.h"
#include "style.h"

int mbtk_style_set_prop(const mbtk_style_t *style, mbtk_styleprop_t *dst, mbtk_kw_t key, mbtk_arg_t value)
{
	if ((style == NULL) || (style->getset_prop == NULL))
		return -1;

	return style->getset_prop(style, dst, key, NULL, &value);
}

int mbtk_style_get_prop(const mbtk_style_t *style, mbtk_styleprop_t *src, mbtk_kw_t key, mbtk_arg_t *dst)
{
	if ((style == NULL) || (style->getset_prop == NULL))
		return -1;

	return style->getset_prop(style, src, key, dst, NULL);
}


int mbtk_display_style_set_prop(mbtk_display_t *dst, mbtk_kw_t key, mbtk_arg_t value)
{
	return mbtk_style_set_prop(dst->style, dst->styleprop, key, value);
}

int mbtk_display_style_get_prop(mbtk_display_t *src, mbtk_kw_t key, mbtk_arg_t *dst)
{
	return mbtk_style_get_prop(src->style, src->styleprop, key, dst);
}

mbtk_kw_t mbtk_kw_bg_color, mbtk_kw_bg_activated_color, mbtk_kw_fg_color, mbtk_kw_frame_width;
mbtk_kw_t mbtk_kw_hilight_color;
mbtk_kw_t mbtk_kw_text_halign, mbtk_kw_text_valign;

void mbtk_style_init(void)
{
	mbtk_kw_bg_color = mbtk_kw("bg.color");
	mbtk_kw_bg_activated_color = mbtk_kw("bg_activated.color");
	mbtk_kw_fg_color = mbtk_kw("fg.color");
	mbtk_kw_hilight_color = mbtk_kw("hilight.color");
	mbtk_kw_frame_width = mbtk_kw("frame.width");
	mbtk_kw_text_halign = mbtk_kw("text.halign");
	mbtk_kw_text_valign = mbtk_kw("text.valign");
}
