/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#ifndef LIBMBTK_WIDGET_CANVAS_NATIVE_H
#define LIBMBTK_WIDGET_CANVAS_NATIVE_H

#include <libmbtk/widget.h>
#include <libmbtk/widget_inlines.h>

/* Subwindow in which the application can draw using native backend calls;
   this makes the application code specific for a given backend but lets the
   application access all the API under the backend. Typical example: opengl
   rendering of 3d scenes, which is impossible through the backend API.
   default handler is ev2cb. */
typedef struct mbtk_canvas_native_s mbtk_canvas_native_t;
struct mbtk_canvas_native_s {
	mbtk_widget_t w;
	mbtk_window_t *subwin;
	int (*minsize)(mbtk_canvas_native_t *c, long *mx, long *my);
	int (*draw)(mbtk_canvas_native_t *c, void *native, long x_offs, long y_offs, long w, long h);
};

mbtk_canvas_native_t *mbtk_canvas_native_new(mbtk_canvas_native_t *caller_alloced);
void mbtk_canvas_native_set_minsize_cb(mbtk_canvas_native_t *c, int (*minsize)(mbtk_canvas_native_t*, long*, long*));
void mbtk_canvas_native_set_draw_cb(mbtk_canvas_native_t *c, int (*draw)(mbtk_canvas_native_t*, void *, long, long, long, long));

/* When the ev2cb handler is installed on the, "event" callbacks are triggered
   on event sunhandled by child widgets. This function binds that callback. */
void mbtk_canvas_native_callback_event(mbtk_canvas_native_t *c, mbtk_event_handled_t (*cb)(mbtk_widget_t *w, mbtk_kw_t id, mbtk_event_t *ev, void *user_data), void *user_data);

MBTK_WIDGET_INLINES(canvas_native)

#endif
