/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#define MBTK_NO_INCLUDE_WIDGETS
#include <libmbtk/widget.h>
#include <libmbtk/widget_imp.h>
#include <libmbtk/draw.h>
#include <libmbtk/widget/Common.h>
#include <libmbtk/widget/hand_toggle.h>

#include <libmbtk/widget/checkbox.h>


static void mbtk_checkbox_imp_destroy(mbtk_widget_t *w) { }

#define FALLBACK_SIZE 10

static int mbtk_checkbox_fallback_minsize(mbtk_widget_t *w, long *mx, long *my)
{
	long l, r, t, b;
	mbtk_window_t *win = w->win;
	const mbtk_style_t *style = mbtk_widget_style(w);
	mbtk_styleprop_t *sp = mbtk_styleprop_get(w);
	mbtk_widget_flags_t flags = w->flags;

	flags.activated = 1;
	style->min_box_frame_size(win, sp, flags, &l, &r, &t, &b);

	w->hvbox.fr_l = l; w->hvbox.fr_r = r;
	w->hvbox.fr_t = t; w->hvbox.fr_b = b;
	*mx = l+r+FALLBACK_SIZE; *my = t+b+FALLBACK_SIZE;

	return 0;
}

static void mbtk_checkbox_fallback_draw(mbtk_widget_t *w)
{
	mbtk_window_t *win = w->win;
	const mbtk_style_t *style = mbtk_widget_style(w);
	mbtk_styleprop_t *sp = mbtk_styleprop_get(w);
	mbtk_widget_flags_t flags = w->flags;
	long l, r, t, b, wi, he;
	long ox = (w->hvbox.w - FALLBACK_SIZE) / 2, oy = (w->hvbox.h - FALLBACK_SIZE) / 2; /* center in parent box */

	flags.activated = 1;
	style->min_box_frame_size(win, sp, flags, &l, &r, &t, &b);
	wi = l + r + FALLBACK_SIZE; he = t + b + FALLBACK_SIZE;

	style->draw_bg_rect(win, sp, flags, w->hvbox.x + ox, w->hvbox.y + oy, wi, he);
	style->draw_box_frame(win, sp, flags, w->hvbox.x + ox, w->hvbox.y + oy, wi, he);

	/* draw the X mark when activated */
	if (w->flags.activated) {
		long x1 = w->hvbox.x + ox + l, y1 = w->hvbox.y + oy + t;
		long x2 = x1 + FALLBACK_SIZE-r, y2 = y1 + FALLBACK_SIZE - b;

		style->set_fg_color(win, sp, w->flags);
		mbtk_draw_line(win, x1, y1, x2, y2);
		mbtk_draw_line(win, x1, y2, x2, y1);
	}
}

void mbtk_checkbox_fallback_draw_overlay(mbtk_widget_t *w)
{
	mbtk_window_t *win = w->win;
	const mbtk_style_t *style = mbtk_widget_style(w);
	mbtk_styleprop_t *sp = mbtk_styleprop_get(w);
	mbtk_widget_flags_t flags = w->flags;
	long l, r, t, b, wi, he;
	long ox = (w->hvbox.w - FALLBACK_SIZE) / 2, oy = (w->hvbox.h - FALLBACK_SIZE) / 2; /* center in parent box */

	flags.activated = 1;
	style->min_box_frame_size(win, sp, flags, &l, &r, &t, &b);
	wi = l + r + FALLBACK_SIZE; he = t + b + FALLBACK_SIZE;

	style->draw_box_overlay(win, sp, flags, w->hvbox.x + ox, w->hvbox.y + oy, wi, he);
}



static const mbtk_widget_imp_t mbtk_checkbox_imp = {
	mbtk_checkbox_imp_destroy,

	mbtk_checkbox_fallback_minsize,
	mbtk_checkbox_fallback_draw,
	mbtk_checkbox_fallback_draw_overlay,

	NULL,  /* get_prop */
	NULL,  /* set_prop */

	NULL, /* post_assign */
	NULL, /* pre_unassign */

	NULL, /* pre_appear */
	NULL, /* post_disappear */

	NULL  /* pre_child_unlink */
};

mbtk_checkbox_t *mbtk_checkbox_new(mbtk_checkbox_t *caller_alloced)
{
	mbtk_checkbox_t *checkbox;
	mbtk_kw_t kw_checkbox;

	mbtk_kw_cache(kw_checkbox, "checkbox");

	mbtk_create_wt(checkbox, kw_checkbox, &mbtk_checkbox_imp, caller_alloced);
	checkbox->w.hvbox.type = HVBOX_HBOX;
	mbtk_handler_toggle_install(&checkbox->w);
	return checkbox;
}

void mbtk_checkbox_callback_toggle(mbtk_checkbox_t *b, mbtk_event_handled_t (*cb)(mbtk_widget_t *w, mbtk_kw_t id, void *user_data), void *user_data)
{
	mbtk_kw_t kw_toggled;
	mbtk_kw_cache(kw_toggled, "toggled");
	mbtk_callback_set_noarg(&b->w, kw_toggled, cb, user_data);
}

