/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#include <stdio.h>
#include <libmbtk/global_typedefs.h>
#include <libmbtk/handler.h>
#define MBTK_NO_INCLUDE_WIDGETS
#include <libmbtk/widget.h>
#include <libmbtk/widget_imp.h>

static const char my_name[] = "hand_root";

typedef struct mbtk_root_handler_s {
	mbtk_handler_t hdr;
} mbtk_root_handler_t;

static mbtk_root_handler_t root_handler;
static int root_handler_inited = 0;


static mbtk_event_handled_t root_event(mbtk_widget_t *widget, mbtk_event_t *ev)
{
	switch(ev->type) {
		case MBTK_EV_KEY_PRESS:
			if (ev->data.key.sym == MBTK_KS_TAB) {
				printf("root tab press!\n");
				return MBTK_EVENT_HANDLED;
			}
			break;
		case MBTK_EV_KEY_RELEASE:
			if (ev->data.key.sym == MBTK_KS_TAB) {
				printf("root tab release!\n");
				return MBTK_EVENT_HANDLED;
			}
			break;
		case MBTK_EV_WIN_CLOSE:
			{
				mbtk_kw_t kw_win_close;

				mbtk_kw_cache(kw_win_close, "win_close");
				return mbtk_callback_noarg(widget, kw_win_close);
			}
		default:;
	}
	return MBTK_EVENT_NOT_HANDLED;
}


void mbtk_handler_root_install(mbtk_widget_t *widget)
{
	if (widget->handler != NULL)
		mbtk_widget_del_handler(widget);

	if (!root_handler_inited) {
		root_handler.hdr.name = my_name;
		root_handler.hdr.event = root_event;
		root_handler_inited = 1;
	}
	widget->handler = &root_handler.hdr;
}
