/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#ifndef LIBMBTK_WIDGET_LABEL_H
#define LIBMBTK_WIDGET_LABEL_H

#include <libmbtk/widget.h>
#include <libmbtk/widget_inlines.h>

/* Renders single or multiple lines of text. */
typedef struct mbtk_label_s {
	mbtk_widget_t w;

	/* configuration */
	mbtk_font_id_t fid;      /* font (style and size) to use for drawing the text */
	signed char rot90;       /* number of times the text is rotated by 90 degrees; positive is CCW */
	unsigned trunc_x:1;      /* truncate in x direction when text is too wide */
	unsigned trunc_y:1;      /* truncate in y direction when text is too high */
	signed halign:2;         /* -1 is left, 0 is center, +1 is right */
	signed valign:2;         /* -1 is top, 0 is center, +1 is bottom */
	unsigned draw_bg:1;      /* whether to draw background rectangle */

	/* internal */
	unsigned widths_valid:1; /* calculated font width values */
	unsigned short numlines; /* total number of lines */
	void *str;               /* opaque line structure of all lines */
	long len;                /* strlen() of the label rendered to plain C string */
	int font_asc, font_dsc;  /* ascend and descend of the font */
} mbtk_label_t;

/* str is always copied in these calls */
mbtk_label_t *mbtk_label_new(mbtk_label_t *caller_alloced, const char *str);
void mbtk_label_set_str(mbtk_label_t *label, const char *str);

/* Always returns a malloc()'d string; the caller needs to free() it. */
char *mbtk_label_get_str(mbtk_label_t *label);

/*** For style code ***/
void mbtk_label_minsize(mbtk_label_t *label, long *mx, long *my);
void mbtk_label_draw(mbtk_label_t *label);


MBTK_WIDGET_INLINES(label)

#endif
