/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#ifndef LIBMBTK_WIDGET_MENU_BAR_H
#define LIBMBTK_WIDGET_MENU_BAR_H

#include <libmbtk/config.h>
#include <libmbtk/widget.h>
#include <libmbtk/widget_inlines.h>
#include <libmbtk/widget/menu.h>
#include <libmbtk/widget/hand_hovradio.h>

/* A HBOX of menu labels, each opening a menu. Layout widget. */

typedef struct mbtk_menu_bar_s {
	mbtk_widget_t w;
	vtp0_t menus;
	mbtk_handler_hovradio_t radio;
} mbtk_menu_bar_t;

/* Create a new, empty menu_bar  */
mbtk_menu_bar_t *mbtk_menu_bar_new(mbtk_menu_bar_t *caller_alloced);

/* Append a menu at the end of the list. The generic version accepts
   any widget for the menu label (should be a label or picture normally); the
   _label version creates the label widget from menu_name. Return
   the label widget appended or NULL on error. */
mbtk_widget_t *mbtk_menu_bar_append(mbtk_menu_bar_t *menu_bar, mbtk_widget_t *label, mbtk_menu_t *menu);
mbtk_widget_t *mbtk_menu_bar_append_label(mbtk_menu_bar_t *menu_bar, const char *menu_name, mbtk_menu_t *menu);

/* Inserts a menu before or after an existing widget/label (appended or
   added earlier to a menu_bar). If ins_before is true, insert before existing,
   else insert after existing. Return the label widget inserted or
   NULL on error.*/
mbtk_widget_t *mbtk_menu_bar_add_at(mbtk_widget_t *existing, mbtk_widget_t *label, mbtk_menu_t *menu, int ins_before);
mbtk_widget_t *mbtk_menu_bar_add_at_label(mbtk_widget_t *existing, const char *menu_name, mbtk_menu_t *menu, int ins_before);



MBTK_WIDGET_INLINES(menu_bar)

#endif
