/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#ifndef LIBMBTK_WIDGET_PANE_H
#define LIBMBTK_WIDGET_PANE_H

#include <libmbtk/config.h>
#include <libmbtk/widget.h>
#include <libmbtk/widget/box.h>
#include <libmbtk/widget_inlines.h>

/* Pane: hbox or vbox that hosts 3 widgets; first an user provided then
   a separator/knob then another user provided. When the pane is larger
   than its minimal size, the user may grab and move the knob to change size
   allocation between the user provided widgets. Composite widget. */

typedef struct mbtk_pane_s {
	mbtk_widget_t w;
	double target_percent; /* keep this percentage for the first widget, if > 0 */
	long target_pixel;     /* or keep this pixel size for the first widget, if > 0 */
	unsigned int horiz:1; /* set to 1 for horizontal layout, 0 for vertical */
	unsigned child_unlink_lock:1; /* internal: set during replacement of a child */
	mbtk_box_t dummy[2];
} mbtk_pane_t;

/* Create a new scrollress bar with min/max set to 0/1, curr set to 0. */
mbtk_pane_t *mbtk_pane_new(mbtk_pane_t *caller_alloced, int horiz);

/* Set curr position of pane separator (really the size allocated for widget 0).
   Clamped so that minsize of each side is respected.  */
void mbtk_pane_set_percent(mbtk_pane_t *pane, double curr);
void mbtk_pane_set_pixel(mbtk_pane_t *pane, long curr);

/* Set or replace child idx=0 (left or top) or child idx=1 (right or bottom).
   With replace, previous content widget is returned but not destroyed; the
   caller needs to destroy it if it is not NULL. With set the previous content
   widget is destroyed. (Initially panes are empty so the first set for each side
   will return NULL.) */
mbtk_widget_t *mbtk_pane_replace_child(mbtk_pane_t *parent, int idx, mbtk_widget_t *w);
void mbtk_pane_set_child(mbtk_pane_t *parent, int idx, mbtk_widget_t *w);

/* Return sub-widget: child.0, sep, child.1 */
mbtk_widget_t *mbtk_pane_subwidget(mbtk_pane_t *pane, mbtk_kw_t subname);


MBTK_WIDGET_INLINES(pane)

#endif
