/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#ifndef LIBMBTK_WIDGET_PIXMAP_H
#define LIBMBTK_WIDGET_PIXMAP_H

#include <libmbtk/config.h>
#include <libmbtk/widget.h>
#include <libmbtk/widget_inlines.h>

/* Button: can be clicked a mouse click release or enter release */
typedef struct mbtk_picture_s {
	mbtk_widget_t w;
	mbtk_kw_t pixmap_kw;
	int pxw, pxh; /* cached pixmap width and height; valid if pxwh_valid is set */
	unsigned pxwh_valid:1;
	mbtk_px_id_t pxid;
} mbtk_picture_t;

/* Create a new picture with pixmap set. Pixmap is named by a keyword;
   pixmap is registered in the given display by the caller prior to the call,
   e.g. using mbtk_mpm_set() or mbtk_xpm_set(). */
mbtk_picture_t *mbtk_picture_new(mbtk_picture_t *caller_alloced, mbtk_kw_t pixmap_kw);

/* Change the pixmap on an existing picture. */
void mbtk_picture_set_pixmap(mbtk_picture_t *pic, mbtk_kw_t pixmap_kw);

MBTK_WIDGET_INLINES(picture)

#endif
