/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#ifndef LIBMBTK_WIDGET_SCROLLED_H
#define LIBMBTK_WIDGET_SCROLLED_H

#include <libmbtk/widget.h>
#include <libmbtk/widget_inlines.h>

/* Scrollable subwindow which hosts a tree of widgets. Composite widget. */
typedef struct mbtk_scrolled_s mbtk_scrolled_t;
struct mbtk_scrolled_s {
	mbtk_widget_t w;
	mbtk_window_t *subwin;
	mbtk_widget_t *root;
};

mbtk_scrolled_t *mbtk_scrolled_new(mbtk_scrolled_t *caller_alloced);

void mbtk_scrolled_set_root_widget(mbtk_scrolled_t *scrolled, mbtk_widget_t *widget);

/* Set the offset of content: content coords of the first (top-left) pixel shown
   in the subwindow. Clamps both cx0 and cy0 to avoid scrolling beyond content.
   Returns 1 if supplied cx0;cy0 made a change. */
int mbtk_scrolled_set_content_origin(mbtk_scrolled_t *scrolled, long cx0, long cy0);

/* Fill in dst with the standard range.val notation: 4 doubles (min, max,
   content top-left offset, content bottom-right offset). If horiz is 1,
   horizontal values are filled in, if 0, vertical values are used. */
void mbtk_scrolled_get_range(mbtk_scrolled_t *scrolled, mbtk_arg_t dst[4], int horiz);

MBTK_WIDGET_INLINES(scrolled)

#endif
