#ifndef LIBMBTK_WIDGET_TABBED_H
#define LIBMBTK_WIDGET_TABBED_H

#include <libmbtk/config.h>
#include <libmbtk/widget.h>
#include <libmbtk/widget_inlines.h>
#include <libmbtk/widget/box.h>
#include <libmbtk/widget/hand_radio.h>

/* Tabbed: show only one of multiple pages depending on which tab is active
   in the selector bar. Composite widget. */

/* where the selector should be; set once on initialization, can not be
   changed later */
typedef enum mbtk_tabbed_layout_e {
	MBTK_TABBED_TOP,
	MBTK_TABBED_LEFT
} mbtk_tabbed_layout_t;

typedef struct mbtk_tabbed_s {
	mbtk_widget_t w;
	mbtk_tabbed_layout_t layout;
	mbtk_box_t selectors, pages;
	mbtk_handler_radio_t radio;
} mbtk_tabbed_t;

/* Create a new tabbed with no title widget. The caller shall create a widget
   (e.g. a hbox with further widgets) and use mbtk_tabbed_set_child() to put
   install it in the tabbed, then mbtk_tabbed_set_title() to attach title widget(s). */
mbtk_tabbed_t *mbtk_tabbed_new(mbtk_tabbed_t *caller_alloced, mbtk_tabbed_layout_t layout);

/* Append a page at the end of the list of pages. The generic version accepts
   any widget for the selector (should be a label or picture normally); the
   _label version creates the label widget for the selector. */
void mbtk_tabbed_append_page(mbtk_tabbed_t *t, mbtk_widget_t *selector, mbtk_widget_t *page);
void mbtk_tabbed_append_page_with_label(mbtk_tabbed_t *t, const char *selector, mbtk_widget_t *page);



/* Return the named subwidget of tabbed:
    - selector is the box in which all tab selectors are
    - pages is the box in which all pages are */
mbtk_widget_t *mbtk_tabbed_subwidget(mbtk_tabbed_t *tabbed, mbtk_kw_t subname);

MBTK_WIDGET_INLINES(tabbed)

#endif
