/*  miniboxtk - small GUI toolkit/widget system - test programs
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <libmbtk/libmbtk.h>
#include <libmbtk/widget.h>
#include <libmbtk/style_f3d.h>
#include <libmbtk/arg_t.h>
#include "../be_init.h"

static char **font;

static mbtk_event_handled_t win_close_cb(mbtk_widget_t *w, mbtk_kw_t id, void *user_data)
{
	mbtk_checkbox_t *chk = user_data;

	if (chk->w.flags.activated)
		return MBTK_EVENT_NOT_HANDLED; /* let the window close */

	printf("Can't close window, check the checkbox first!\n");
	return MBTK_EVENT_HANDLED; /* do not close the window */
}

static mbtk_event_handled_t force_close(mbtk_widget_t *w, mbtk_kw_t id, void *user_data)
{
	mbtk_window_t *win = w->win;
	mbtk_window_close(win);
	return 0;
}


int main(int argc, char *argv[])
{
	mbtk_window_t *win;
	mbtk_backend_t be;
	mbtk_display_t disp;
	mbtk_box_t *hbox;
	mbtk_label_t *wl;
	mbtk_checkbox_t *chk;
	mbtk_button_t *btn;

	font = tests_backend_init(&be, &disp, &argc, &argv);
	mbtk_f3d_style(&disp);

	disp.default_fid = mbtk_resolve_font(&disp, 1, font);

	win = mbtk_window_new(&disp, MBTK_WNTY_NORMAL, "miniboxtk conditional exit", -1, -1, 400, 100);
	if (win == NULL) {
		fprintf(stderr, "Unable to create top window\n");
		exit(1);
	}

	hbox = mbtk_hbox_new(NULL);
	mbtk_window_set_root_widget(win, &hbox->w);

	wl = mbtk_label_new(NULL, "Check this if\nready to close:");
	mbtk_box_add_widget(hbox, &wl->w, 0);

	chk = mbtk_checkbox_new(NULL);
	mbtk_box_add_widget(hbox, &chk->w, 0);

	btn = mbtk_button_new_with_label("Force-close");
	mbtk_box_add_widget(hbox, &btn->w, 0);
	mbtk_button_callback_click(btn, force_close, NULL);

	/* get win_close_cb called back on window close request, pass the checkbox
	   widget so the callback function can look at checkbox state */
	mbtk_window_callback_close(win, win_close_cb, chk);


	/* main loop, wait until the window is actually closed */
	mbtk_run_window(win);

	/* automatically closes all windows, displays and backends */
	mbtk_uninit();
}
